unit UGetGeoCode;

//=========================================================================
//
// I N T E R F A C E
//
//=========================================================================
interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, IdBaseComponent, IdComponent,
  IdTCPConnection, IdTCPClient, IdHTTP, Grids, ValEdit, ComCtrls, Buttons;

type

  TThreadGeoCode = class(TThread)
  private
    FDireccion: string;
    FAllText: TStrings;
    FGeoStatusCode: Integer;
    FGeoAddress: string;
    FGeoCountryCode: string;
    FGeoCountryName: string;
    FGeoAdminArea: string;
    FGeoAddressLine: string;
    FGeoLocalityName: string;
    FGeoPostalCode: string;
    FGeoAccuracy: integer;
    FGeoLatitud: string;
    FGeoLongitud: string;

    procedure _ExtractResult();
    function _StatusCodeToStr(ACode:Integer):string;
    function _AccuracyToStr(ACode:integer):string;
  protected
    procedure Execute; override;
  public
    destructor Destroy; override;
    constructor Create(ADireccion:string;
                       ThreadPriority:TThreadPriority=tpNormal);

    // Direccion a buscar
    property Direccion:string read FDireccion write FDireccion;
    // Salida
    property AllText:TStrings read FAllText write FAllText;
    // propiedades de posicin
    property GeoStatusCode:Integer read FGeoStatusCode write FGeoStatusCode;
    property GeoAddress:string read FGeoAddress write FGeoAddress;
    property GeoCountryName:string read FGeoCountryName write FGeoCountryName;
    property GeoCountryCode:string read FGeoCountryCode write FGeoCountryCode;
    property GeoAdminArea:string read FGeoAdminArea write FGeoAdminArea;
    property GeoAddressLine:string read FGeoAddressLine write FGeoAddressLine;
    property GeoLocalityName:string read FGeoLocalityName write FGeoLocalityName;
    property GeoPostalCode:string read FGeoPostalCode write FGeoPostalCode;
    property GeoAccuracy:integer read FGeoAccuracy write FGeoAccuracy;
    property GeoLatitud:string read FGeoLatitud write FGeoLatitud;
    property GeoLongitud:string read FGeoLongitud write FGeoLongitud;

  published

  end;

  TFormMapa = class(TForm)
    pnlCentral: TPanel;
    mmWeb: TMemo;
    Panel1: TPanel;
    edt_edtTexto: TEdit;
    vle1: TValueListEditor;
    Splitter1: TSplitter;
    stat1: TStatusBar;
    lbl1: TLabel;
    btn2: TButton;
    btn1: TSpeedButton;
    mm1: TMemo;
    edtTexto: TComboBox;
    procedure edt_edtTextoKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure btn1Click(Sender: TObject);
    procedure btn2Click(Sender: TObject);
    procedure edtTextoChange(Sender: TObject);
    procedure stat1Click(Sender: TObject);
  private
    _ViewGoogle:Boolean;

    procedure ThreadTerminated(Sender:TObject);
  published
  end;

var
  FormMapa: TFormMapa;

//=========================================================================
//
// I M P L E M E N T A T I O N
//
//=========================================================================
implementation

{$R *.dfm}

uses
  StrUtils, ShellAPI;

procedure TFormMapa.edt_edtTextoKeyDown(Sender: TObject; var Key: Word;  Shift: TShiftState);
const
  STR_WEB = 'http://maps.google.com/maps/geo?q=';
var
  Stream: TStringStream;
  Str, res:string;
  IdHTTP1: TIdHTTP;
begin

  if (key <> VK_RETURN) then begin
    Exit;
  end;

  btn2Click(Self);
  Exit;

  // ini
  Stream :=  TStringStream.Create(res);
  IdHTTP1 := TIdHTTP.Create(nil);

  // proteccion para liberar
  try

    Str := STR_WEB + edtTexto.Text;
    idhttp1.Get(Str, Stream);
    mmWeb.Lines.Text := Stream.DataString;

  finally
    FreeAndNil(IdHTTP1);
    FreeAndNil(Stream);
  end;

end;




procedure TFormMapa.btn1Click(Sender: TObject);
begin
  // levantar el flag
  _ViewGoogle := True;
  // Calcular
  btn2Click(nil);

end;

{ TThreadGeoCode }
constructor TThreadGeoCode.Create(ADireccion:string;
                                  ThreadPriority: TThreadPriority);
begin
   inherited Create(True);
   Self.FAllText := TStringList.Create();
   Self.FDireccion := ADireccion;

end;

destructor TThreadGeoCode.Destroy();
begin
  if Assigned(Self.FAllText) then begin
    FreeAndNil(Self.FAllText);
  end;

  inherited;
end;

procedure TThreadGeoCode.Execute();
const
  STR_WEB = 'http://maps.google.com/maps/geo?q=';
var
  Stream: TStringStream;
  Str, res:string;
  IdHTTP1: TIdHTTP;
begin

  // ini
  Stream :=  TStringStream.Create(res);
  IdHTTP1 := TIdHTTP.Create(nil);

  // proteccion para liberar
  try

    Str := STR_WEB + Self.FDireccion;
    idhttp1.Get(Str, Stream);
    Self.AllText.Text := Stream.DataString;

    _ExtractResult();


  finally
    FreeAndNil(IdHTTP1);
    FreeAndNil(Stream);
  end;


end;

procedure TFormMapa.btn2Click(Sender: TObject);
var
  th:TThreadGeoCode;
  str:string;
begin

  
  edtTexto.Text := AnsiReplaceText(edtTexto.Text, ' ', '+');

  th := TThreadGeoCode.Create(edtTexto.Text);
  th.OnTerminate := ThreadTerminated;
  th.FreeOnTerminate := True;

  vle1.Strings.Clear;
  th.Resume;
end;


procedure TFormMapa.ThreadTerminated(Sender: TObject);
var
  th:TThreadGeoCode;
  Str:string;
begin

  th := TThreadGeoCode(Sender);

  vle1.Strings.Add('Direccin=' + th.Direccion);
  vle1.Strings.Add('StatusCode=' + IntToStr(th.GeoStatusCode));
  vle1.Strings.Add('   Desc.=' + th._StatusCodeToStr(th.GeoStatusCode));
  vle1.Strings.Add('Address=' + th.GeoAddress);
  vle1.Strings.Add('CountryName=' + th.GeoCountryName);
  vle1.Strings.Add('CountryCode=' + th.GeoCountryCode);
  vle1.Strings.Add('AdminArea=' + th.GeoAdminArea);
  vle1.Strings.Add('AddressLine=' + th.GeoAddressLine);
  vle1.Strings.Add('Localidad=' + th.GeoLocalityName);
  vle1.Strings.Add('Cod.postal=' + th.GeoPostalcode);
  vle1.Strings.Add('Accuracy=' + IntToStr(th.GeoAccuracy));
  vle1.Strings.Add('Latitud=' + th.GeoLatitud);
  vle1.Strings.Add('Longitud=' + th.GeoLongitud);

  mmWeb.Lines.Text := th.FAllText.Text;


  // Visualizar en Google Maps
  if (_ViewGoogle) then begin
    _ViewGoogle := False;

    Str := Format('http://maps.google.es/maps?&q=%s,%s&z=15',
      [th.GeoLatitud, th.GeoLongitud]);

    ShellExecute(Handle,
         'open',
         PChar(Str),
         nil,
         nil,
         SW_SHOW);
  end;

end;


function TThreadGeoCode._StatusCodeToStr(ACode: integer): string;
begin

  case ACode of
    200: Result := 'G_GEO_SUCCESS';
    400: Result := 'G_GEO_BAD_REQUEST';
    500: Result := 'G_GEO_SERVER_ERROR';
    601: Result := 'G_GEO_MISSING_QUERY';
    //601: Result := 'G_GEO_MISSING_ADDRESS';
    602: Result := 'G_GEO_UNKNOWN_ADDRESS';
    603: Result := 'G_GEO_UNAVAILABLE_ADDRESS';
    604: Result := 'G_GEO_UNKNOWN_DIRECTIONS';
    610: Result := 'G_GEO_BAD_KEY';
    620: Result := 'G_GEO_TOO_MANY_QUERIES';
  else
    Result := 'unknow code';
  end;

end;

function TThreadGeoCode._AccuracyToStr(ACode: Integer): string;
begin

  case ACode of
    0: 	Result := 'Ubicacin desconocida.';
    1:  Result := 'Precisin a nivel de pas.';
    2: 	Result := 'Precisin a nivel de regin';
    3: 	Result := 'Precisin a nivel de subregin';
    4: 	Result := 'Precisin a nivel de ciudad o pueblo.';
    5:  Result := 'Precisin a nivel de cdigo postal.';
    6: 	Result := 'Precisin a nivel de calle.';
    7: 	Result := 'Precisin a nivel de interseccin.';
    8: 	Result := 'Precisin a nivel de direccin.';
  else
    Result := 'unknow code';
  end;

end;

procedure TThreadGeoCode._ExtractResult();
const
  STR_STATUS            = '"Status":';
  STR_CODESTATUS        = '"code":';
  STR_ADDRESS           = '"address":';
  STR_COUNTRYNAME       = '"CountryNameCode":';
  STR_COUNTRYCODE       = '"CountryName":';
  STR_ADMINAREA         = '"AdministrativeAreaName":';
  STR_ADDRESSLINE       = '"AddressLine":';
  STR_LOCALITYNAME      = '"LocalityName":';
  STR_POSTALCODENUMBER  = '"PostalCodeNumber":';
  STR_ACCURACY          = '"Accuracy":';
  STR_COORDINATES       = '"coordinates": [';
var
  value:Integer;
  Str:String;
  TS:TStrings;

  function __GetData(AStr:string; EndChar:string=''):string;
  var
    p:Integer;
    i,j:Integer;
    Str:string;
  begin
    Result := '';
    for i := 0 to (Self.FAllText.Count - 1) do begin
      Str := Self.FAllText[i];
      p := AnsiPos(AStr, Str);

      if (p > 0) then begin
        Result := Copy(Str, p + Length(AStr), Length(Str));
        Break;
      end;
    end;

    if (EndChar <> '') then begin
      j := AnsiPos(EndChar, Result);
      if (j > 0) then begin
        Result := Copy(Result, 1, j-1);
      end;
    end;
  end;
  //

begin

  // Obtener cdigo
  Str := __GetData(STR_CODESTATUS);
  Str := AnsiReplaceText(Str, ',', '');
  Self.FGeoStatusCode := StrToIntDef(Str, -1);

  // Error
  if (Self.GeoStatusCode <> 200) then begin
    // DEVOLVER EL STRING
    Exit;
  end;

  // Continuar...
  // Obtener direccion
  Str := __GetData(STR_ADDRESS);
  Str := AnsiReplaceText(Str, '"', '');
  Self.FGeoAddress := Trim(Str);
  // Country Name
  Str := __GetData(STR_COUNTRYNAME, ',');
  Str := AnsiReplaceText(Str, '"', '');
  Self.FGeoCountryName := Trim(Str);
  // CountryCode
  Str := __GetData(STR_COUNTRYCODE, ',');
  Str := AnsiReplaceText(Str, '"', '');
  Self.FGeoCountryCode := Trim(Str);
  // AdminArea
  Str := __GetData(STR_ADMINAREA, ',');
  Str := AnsiReplaceText(Str, '"', '');
  Self.FGeoAdminArea := Trim(Str);
  // addressLine
  Str := __GetData(STR_ADDRESSLINE, ',');
  Str := AnsiReplaceText(Str, '"', '');
  Str := AnsiReplaceText(Str, '[', '');
  Str := AnsiReplaceText(Str, ']', '');
  Str := AnsiReplaceText(Str, '}', '');
  Self.FGeoAddressLine := Trim(Str);
  // Localidad
  Str := __GetData(STR_LOCALITYNAME, ',');
  Str := AnsiReplaceText(Str, '"', '');
  Self.FGeoLocalityName := Trim(Str);
  // Cod. Postal
  Str := __GetData(STR_POSTALCODENUMBER, ',');
  Str := AnsiReplaceText(Str, '"', '');
  Str := AnsiReplaceText(Str, '}', '');
  Self.FGeoPostalCode := Trim(Str);
  // Exactitud
  Str := __GetData(STR_ACCURACY, '}');
  Str := Trim(Str);
  Self.FGeoAccuracy := StrToIntDef(Str, -1);
  // Coordenadas
  Str := __GetData(STR_COORDINATES, ']');
  Str := AnsiReplaceText(Str, '"', '');
  TS := TStringList.Create();
  try
    TS.Delimiter := ',';
    TS.DelimitedText := Trim(Str);
    if (TS.Count > 1) then
      Self.FGeoLongitud := TS[0];
    if (TS.Count > 2) then
      Self.FGeoLatitud := TS[1];
  finally
    FreeandNil(TS);
  end;


end;

procedure TFormMapa.edtTextoChange(Sender: TObject);
begin
  if (edtTexto.Text <> '') then begin
    Self.btn2Click(nil);
  end;

end;

procedure TFormMapa.stat1Click(Sender: TObject);
begin
  ShellExecute(Handle,
       'open',
       'http://neftali.clubdelphi.com/',
       nil,
       nil,
       SW_SHOW);
end;

end.
